package com.blazinggames.tmod
{
	import com.blazinggames.tmod.common.UITools;
	
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.geom.Rectangle;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.utils.getTimer;
	
	public class GameEpisode extends MovieClip
	{
// ***** CONSTANTS *****
		
		public static const GAME_OVER_EVENT:String = "GameOver";
		public static const GAME_WON_EVENT:String = "GameWon";
		private static const UPDATE_MS_CAP:int = 200;
		public static const DEFAULT_MS_PER_INTERVAL:int = 10;
		
		// Standard game states (some reserved for future needs, more can be appended)
		public static const GAMESTATE_STARTING:int = 0;
		public static const GAMESTATE_PRE_PLAY:int = 1; // reserved
		public static const GAMESTATE_PLAYING:int = 2;
		public static const GAMESTATE_PLAYING_RESERVED1:int = 3;
		public static const GAMESTATE_PLAYING_RESERVED2:int = 4;
		public static const GAMESTATE_PLAYING_RESERVED3:int = 5;		
		public static const GAMESTATE_LOST_PLAYER:int = 6;
		public static const GAMESTATE_EVENT_RESERVED1:int = 7;
		public static const GAMESTATE_EVENT_RESERVED2:int = 8;
		public static const GAMESTATE_LEVEL_COMPLETE:int = 9;
		public static const GAMESTATE_GAME_OVER:int = 10;
		public static const GAMESTATE_GAME_WON:int = 11;

// ***** VARIABLES *****
		
		// state and overlay control
		public var _playerInfo:PlayerInfo;
		protected var _overlayMessage:TextField;
		protected var _gameState:int;

		// interval management
		protected var _lastUpdateTime:int;
		protected var _intervalRate:int;
	
// ***** CONSTRUCTOR *****

		public function GameEpisode()
		{
			super();
			_intervalRate = 10;
			_overlayMessage = UITools.buildTextField(
						new Rectangle(0,225,640,150),
						new TextFormat("_sans",24,0xFFCC00,true,null,null,null,null,"center"),true);
		}
		
// ***** METHODS - Overlay Management *****
		
		public function showOverlay(message:String):void
		{
			// Make sure overlay is the top layer by re-adding it
			if (contains(_overlayMessage))
				removeChild(_overlayMessage);
			
			_overlayMessage.text = message;
			addChild(_overlayMessage);
			_overlayMessage.visible = true;			
		}
		
		public function hideOverlay():void
		{
			if (contains(_overlayMessage))
				removeChild(_overlayMessage);
		}
		
// ***** METHODS - Game State Management *****

		public function startGame(playerInfo:PlayerInfo):void
		{
			// reset game
			_playerInfo = playerInfo;
			_playerInfo.lives = 3;
			_playerInfo.score = 0;
			_playerInfo.level = 0;

			stage.addEventListener(MouseEvent.MOUSE_UP, handleMouseUp);
			_lastUpdateTime = getTimer();
			stage.addEventListener(Event.ENTER_FRAME, handleEnterFrame);
			changeGameState(GAMESTATE_STARTING);
		}

		public function stopGame()
		{
			stage.removeEventListener(MouseEvent.MOUSE_UP, handleMouseUp);
			stage.removeEventListener(Event.ENTER_FRAME, handleEnterFrame);
		}
		
		protected function clearState(oldState:int):void
		{
			hideOverlay();
		}
		
		protected function startState(newState:int):void
		{
			if (newState == GAMESTATE_PLAYING) {
				if (_playerInfo.gameMode == PlayerInfo.STORY_GAME)
					dispatchEvent(new Event(GAME_WON_EVENT));
				else
					dispatchEvent(new Event(GAME_OVER_EVENT));
			} else
				showOverlay("GAME NOT IMPLEMENTED!\nClick to exit...");
		}
		
		public function changeGameState(state:int)
		{
			clearState(_gameState);
			_gameState = state;
			startState(_gameState);
		}

// ***** METHODS - event handlers *****
		
		public function handleMouseUp(evt:MouseEvent)
		{
			changeGameState(GAMESTATE_PLAYING);
		}
				
		protected function handleEnterFrame(event:Event):void
		{
/*			
			if (_gameState != GAMESTATE_PLAYING) {
				_lastUpdateTime = getTimer();
				return;
			}
*/			
			var curTime:int = getTimer();
			var delta = curTime - _lastUpdateTime;
			if (delta > UPDATE_MS_CAP) {
				delta = UPDATE_MS_CAP;
				_lastUpdateTime = curTime - delta;
			}
			var numIntervals:int = delta / _intervalRate;
			_lastUpdateTime += (numIntervals * _intervalRate);
			//						trace("Number of intervals is " + numIntervals);
			var startingState = _gameState;
			for (var cntr:int = 0; ((cntr < numIntervals) && (_gameState == startingState)); ++cntr)
				updateWorld();			
		}
		
		public function updateWorld():void
		{
		}
	}
}