﻿package com.blazinggames.tmod.common
{
	import flash.display.BitmapData;
	import flash.display.BlendMode;
	import flash.display.DisplayObject;
	import flash.display.Stage;
	import flash.geom.ColorTransform;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.text.TextField;
	import flash.text.TextFormat;

	public class UITools
	{

		public static const RAD_TO_DEGREE:Number = 180 / Math.PI;
		public static const DEGREE_TO_RAD:Number = Math.PI / 180;
			
		public function UITools()
		{
		}

		public static function buildTextField(	bounds:Rectangle,
											format:TextFormat,
											multiline:Boolean = false,
											isSolid:Boolean = false,
											isBordered:Boolean = false,
											backColor:uint = 0xCC000000,
											borderColor:uint = 0xFFFFFFFF):TextField
		{
			var tf:TextField = new TextField();
			tf.x = bounds.x;
			tf.y = bounds.y;
			tf.width = bounds.width;
			tf.height = bounds.height;
			tf.defaultTextFormat = format;
			tf.multiline = multiline;
			tf.selectable = false;
			tf.background = isSolid;
			tf.backgroundColor = backColor;
			tf.border = isBordered;
			tf.borderColor = borderColor;
			return tf;
		}
	
		public static function perPixelBoundCheck(	d1:DisplayObject, 
													d2:DisplayObject):Boolean
		{
			// find the stage locations
			var ref:Stage = d1.stage;
			var bound1:Rectangle = d1.getBounds(ref);
			var bound2:Rectangle = d2.getBounds(ref);
			var intersect:Rectangle = bound1.intersection(bound2);
			if (intersect.width == 0)
				return false;
						
			// grab and translate matrixes for bitmap
			var d1Matrix:Matrix = d1.transform.concatenatedMatrix.clone();
			d1Matrix.translate(-intersect.x, -intersect.y);
			var d2Matrix:Matrix = d2.transform.concatenatedMatrix.clone();
			d2Matrix.translate(-intersect.x, -intersect.y);
			
			// now draw display objects onto bitmap into channel
			var bitmapData:BitmapData = new BitmapData(Math.ceil(intersect.width), Math.ceil(intersect.height),false,0);
			var colorTransform:ColorTransform = new ColorTransform(1,1,1,1,0,0,0,128);
			colorTransform.color = 0xFF00FF;
			bitmapData.draw(d1, d1Matrix, colorTransform, BlendMode.NORMAL);
			colorTransform.color = 0xFF00;
			bitmapData.draw(d2, d2Matrix, colorTransform, BlendMode.ADD);

			var hitRect:Rectangle = bitmapData.getColorBoundsRect(0xffffffff,0xffffffff);
			bitmapData.dispose();
			return (hitRect.width > 0);
		}
		
		public static function findAngle(srcX:Number, srcY:Number, 
										 targetX:Number, targetY:Number, 
										 inRads:Boolean = true):Number
		{
			var rads:Number = Math.atan2(targetY - srcY, targetX - srcX);
			if (inRads)
				return rads;
			else
				return rads * RAD_TO_DEGREE;
		}
		
		public static function findObjectAngle(src:Object, target:Object, inRads:Boolean = true):Number
		{
			return findAngle(src.x, src.y, target.x, target.y, inRads);	
		}
		
		public static function findDistanceSquared(obj1X:Number, obj1Y:Number, obj2X:Number, obj2Y:Number):Number
		{
			var deltaX:Number = obj1X - obj2X;
			var deltaY:Number = obj1Y - obj2Y;
			return (deltaX * deltaX + deltaY * deltaY);
		}
	}
}