package com.blazinggames.tmod.common.arcade
{
	import flash.display.MovieClip;
	
	/**
	 * Container stub-class for a heart movieclip that will be used by the
	 * HeartBar class for arcade status bars to indicate how healthy the 
	 * player is. Heart movies need to implement the following labels that
	 * reflect transitions and states:
	 * normal - HEART_NORMAL state is the full-life indicator
	 * losing - HEART_LOSING Animated loss of heart with transition to lost state
	 * lost - HEART_LOST reflects a missing life
	 * gaining - HEART_GAIN regaining health
	 * 
	 * It should be noted that the movie symbol does not have to be a heart
	 * but can be any symbol that reflects the lives of the player.
	 */   
	public class Heart 
	extends MovieClip
	implements ILifeIcon
	{
// ***** CONSTANTS *****
		
		public static const HEART_NORMAL:int = 0;
		public static const HEART_LOSING:int = 1;
		public static const HEART_LOST:int = 2;
		public static const HEART_GAIN:int = 3;

		private static const LABEL_NORMAL:String = "normal";
		private static const LABEL_LOSING:String = "losing";
		private static const LABEL_LOST:String = "lost";
		private static const LABEL_GAIN:String = "gaining";

// ***** VARIABLES *****
		
		private var heartState:int;
		
// ***** CONSTRUCTOR *****

		public function Heart()
		{
			super();
			heartState = HEART_NORMAL;
		}
		
		public function setState(state:int):void
		{
			switch (state) {
				case HEART_LOSING:
					gotoAndPlay(LABEL_LOSING);
					heartState = HEART_LOST;
					break;
				case HEART_LOST:
					gotoAndPlay(LABEL_LOST);
					heartState = HEART_LOST;
					break;
				case HEART_GAIN:
					gotoAndPlay(LABEL_GAIN);
					heartState = HEART_NORMAL;
					break;
				default: // HEART_NORMAL:
					gotoAndPlay(LABEL_NORMAL);
					heartState = HEART_NORMAL;
					break;
			}
		}
		
		public function killHeart():void
		{
			if (heartState != HEART_LOST)
				setState(HEART_LOSING);
		}

		public function reviveHeart():void
		{
			if (heartState != HEART_NORMAL)
				setState(HEART_GAIN);
		}
		
		public function isAlive():Boolean
		{
			if ((heartState == HEART_GAIN) || (heartState == HEART_NORMAL))
				return true;
			else
				return false;
		}
		
		public function endLife():void
		{
			killHeart();
		}
		
		public function startLife():void
		{
			reviveHeart();
		}
		
	}
}