package com.blazinggames.tmod.common.arcade
{
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.geom.Point;
	import flash.utils.getTimer;
	
	/**
	 * Laser blast for fake 3D games to fake the appearance of a laser being
	 * shot and an incoming laser (so 2 separate classes doing the same thing
	 * are'nt needed). As this uses a movieClip for the assets, the following
	 * labelled sequences that last 1/2 second are needed:
	 * 	normal - blank keyframe for when no laser activity (looped)
	 * 	crosshair - your typical targetting crosshair for the player (looped)
	 * 	shooting - player taking a shot
	 * 	shot - player being shot
	 */
	public class LaserFake3D extends MovieClip
	{
// ***** CONSTANTS *****

		public static const LASER_FINISHED_EVENT:String = "LaserDone";
		public static const LASER_TIME_MS:int = 500;
		
		private static const LABEL_NORMAL:String = "normal";
		private static const LABEL_CROSSHAIR:String = "crosshair";
		private static const LABEL_SHOOTING:String = "shooting";
		private static const LABEL_BEING_SHOT:String = "shot";
		private static const MODE_NORMAL:int = 0;
		private static const MODE_CROSSHAIR:int = 1;
		private static const MODE_SHOOTING:int = 2;
		private static const MODE_BEING_SHOT:int = 3;
		
// ***** VARIABLES *****
		
		private var _mode:int;
		private var _endMode:int;
		private var _laserFrom:Point;
		private var _laserTo:Point;
		private var _startTime:int;
		
// ***** CONSTRUCTORS *****

		public function LaserFake3D()
		{
			super();
		}
		
		public function makeCrosshair(b:Boolean)
		{
			_mode = MODE_CROSSHAIR;
			gotoAndPlay(LABEL_CROSSHAIR);
		}
		
		public function fire(from:Point, to:Point, shooting:Boolean = true)
		{
			_startTime = getTimer();
			x = from.x;
			y = from.y;
			_laserFrom = from;
			_laserTo = to;
			_endMode = _mode;
			if (shooting) {
				_mode = MODE_SHOOTING;
				gotoAndPlay(LABEL_SHOOTING);
			} else {
				_mode = MODE_BEING_SHOT;
				gotoAndPlay(LABEL_BEING_SHOT);				
			}
			stage.addEventListener(Event.ENTER_FRAME, handleShooting);
		}
		
		protected function handleShooting(event:Event):void
		{
			var delta:int = Math.min(LASER_TIME_MS, getTimer() - _startTime);
			var fraction:Number = Number(delta) / Number(LASER_TIME_MS);
			x = _laserFrom.x + (_laserTo.x - _laserFrom.x) * fraction;
			y = _laserFrom.y + (_laserTo.y - _laserFrom.y) * fraction;
			if (delta == LASER_TIME_MS) {
				stage.removeEventListener(Event.ENTER_FRAME, handleShooting);
				if (_endMode == MODE_CROSSHAIR) {
					_mode = MODE_CROSSHAIR;
					gotoAndPlay(LABEL_CROSSHAIR);
				} else {
					_mode = MODE_NORMAL;
					gotoAndPlay(LABEL_NORMAL);
				}
				var evt:Event = new Event(LASER_FINISHED_EVENT);
				dispatchEvent(evt);
			}
		}
	}
}