package com.blazinggames.tmod.common.arcade
{
	import flash.display.DisplayObject;
	import flash.display.MovieClip;
	
	public class LifeBar extends MovieClip
	{
// ***** CONSTANTS *****
		
		public static const ALIGN_LEFT:int = 0;
		public static const ALIGN_CENTER:int = 1;
		public static const ALIGN_RIGHT:int = 2;
		
// ***** VARIABLES *****
		
		private var icons:Array;
		private var health:int;
		
		private var barWidth:Number;
		private var barHeight:Number;
		private var iconWidth:Number;
		private var iconHeight:Number;

		private var lastLayoutIconCount:int;
		
// ***** CONSTRUCTOR *****
		private var align:int;

		public function LifeBar(w:Number = 320, h:Number = 64, iconW:Number = 64, iconH:Number=64)
		{
			super();
			icons = new Array();
			health = 0;
			lastLayoutIconCount = 0;
			barWidth = w;
			barHeight = h;
			iconWidth = iconW;
			iconHeight = iconH;
		}

// ***** METHODS - Bar Manipulation *****
		
		public function add(icon:ILifeIcon, addHealth:Boolean = true):void
		{
			if (addHealth)
				++health;
			icons.push(icon);
			addChild(DisplayObject(icon));
			updateLayout();
		}
			
		public function remove(removeHealth:Boolean = true):ILifeIcon
		{
			var icon:ILifeIcon = icons.pop();
			if (icon != null)
				removeChild(DisplayObject(icon));
			if (removeHealth)
				--health;
			updateLayout();
			return icon;
		}
		
		public function numIcons():int
		{
			return icons.length;
		}
		
		public function setHealth(health:int):void
		{
			this.health = health;
			updateLayout();
		}
		
		public function getHealth():int
		{
			return health;
		}
		
		public function increaseHealth():int
		{
			++health;
			updateLayout();
			return health;
		}
		
		public function decreaseHealth():int
		{
			--health;
			updateLayout();
			return health;
		}
		
// ***** METHODS - Layout *****
		
		public function setAlignment(alignment:int):void
		{
			align = alignment;
			performLayout();
		}
		
		/** used to lay out bar. Is separated from updateLayout to make it
		 * easier to change how layout works in a subclass
		*/
		protected function performLayout():void
		{
			var adjustedIconWidth:Number = iconWidth;
			var adjustedIconHeight:Number = iconHeight;
			var numRows:int = barHeight / adjustedIconHeight;
			var iconsPerRow:int = barWidth / adjustedIconWidth;
			var iconCount:int = iconsPerRow * numRows;
			
			while (iconCount < icons.length) {
				adjustedIconWidth /= 2;
				adjustedIconHeight /= 2;
				numRows = barHeight / adjustedIconHeight;
				iconsPerRow = barWidth / adjustedIconWidth;
				iconCount = iconsPerRow * numRows;
			}
			
			// place the icons
			var numIcons:int = icons.length;
			var startX:Number = 0;
			var xStep:Number = adjustedIconWidth;
			if (align == ALIGN_CENTER) {
				if (numIcons < iconsPerRow)
					startX = barWidth / 2 - numIcons * adjustedIconWidth / 2;
				else
					startX = (barWidth - iconsPerRow * adjustedIconHeight) / 2;
			} else if (align == ALIGN_RIGHT) {
				startX = barWidth - adjustedIconWidth;
				xStep = -adjustedIconWidth; 
			}
			var iconX:Number = startX;
			var iconY:Number = 0;
			var rowCount:int = 0;
			for (var cntr:int = 0; cntr < numIcons; ++cntr) {
				icons[cntr].x = iconX;
				icons[cntr].y = iconY;
				icons[cntr].width = adjustedIconWidth;
				icons[cntr].height = adjustedIconHeight;
				iconX += xStep;
				++rowCount;
				if (rowCount >= iconsPerRow) {
					rowCount = 0;
					iconY += adjustedIconHeight;
					iconX = startX;
					if ((align == ALIGN_CENTER) && ((numIcons-cntr) < iconsPerRow )) {
						iconX = barWidth / 2 - (numIcons-cntr) * adjustedIconWidth / 2;
					}
				}
			}
		}
		
		protected function updateLayout():void
		{
			// lay out the icons if number of icons has changed
			if (icons.length != lastLayoutIconCount) {
				lastLayoutIconCount = icons.length;
				performLayout();
			}
			
			// set health
			var numIcons:int = icons.length;
			var curHealth:int = Math.max(0, Math.min(health, numIcons));
			var cntr:int;
			
			for (cntr = 0; cntr < curHealth; ++cntr)
				if (icons[cntr].isAlive() == false)
					icons[cntr].startLife();
			for (cntr = curHealth; cntr < numIcons; ++cntr)
				if (icons[cntr].isAlive())
					icons[cntr].endLife();
		}

	}
}