package com.blazinggames.tmod.common.data
{
	/**
	 * Pools are used to hold lists of items that are itterated through
	 * with some items needing to be moved to different pools. The general
	 * use is to have a main pool and a dead pool with the dead pool objects
	 * being re-used when new items are needed. To make pool management easier
	 * they need to be able to reset themselves to a reference state and
	 * make copies of themselves (as they may be the reference object that
	 * new items are generated from).
	 * 
	 * @author Billy D. Spelchan
	 */
	public interface IPoolMember
	{
		/** 
		 * Sets the ID of the pool this item should be in.
		 * 
		 * @param id numeric identifier of the pool item should be in.
		 */
		function changePool(id:int):void;
		
		/**
		 * Returns the id for the pool the item is currently in.
		 */
		function getCurrentPool():int;
		
		/**
		 * Creates a copy of itself. The extent of this copy is class 
		 * dependent.
		 * @ returns Object that is a copy of self.
		 */
		function createCopy():IPoolMember;
		
		/**
		 * Sets state to that of the source object.
		 */
		function copyState(source:IPoolMember):void;
	}
}