package com.blazinggames.tmod.common.map
{
	public class FlagMap
	{
// ***** CONSTANTS *****
		
		public static const BIT_MASKS:Array = new Array (
			0x00000001, 0x00000002, 0x00000004, 0x00000008, 0x00000010, 0x00000020, 0x00000040, 0x00000080,
			0x00000100, 0x00000200, 0x00000400, 0x00000800, 0x00001000, 0x00002000, 0x00004000, 0x00008000,
			0x00010000, 0x00020000, 0x00040000, 0x00080000, 0x00100000, 0x00200000, 0x00400000, 0x00800000,
			0x01000000, 0x02000000, 0x04000000, 0x08000000, 0x10000000, 0x20000000, 0x40000000, 0x80000000
		);
		
		public static const INVERT_MASKS:Array = new Array (
			0xFFFFFFFE, 0xFFFFFFFD, 0xFFFFFFFB, 0xFFFFFFF7, 0xFFFFFFEF, 0xFFFFFFDF, 0xFFFFFFBF, 0xFFFFFF7F,
			0xFFFFFEFF, 0xFFFFFDFF, 0xFFFFFBFF, 0xFFFFF7FF, 0xFFFFEFFF, 0xFFFFDFFF, 0xFFFFBFFF, 0xFFFF7FFF,
			0xFFFEFFFF, 0xFFFDFFFF, 0xFFFBFFFF, 0xFFF7FFFF, 0xFFEFFFFF, 0xFFDFFFFF, 0xFFBFFFFF, 0xFF7FFFFF,
			0xFEFFFFFF, 0xFDFFFFFF, 0xFBFFFFFF, 0xF7FFFFFF, 0xEFFFFFFF, 0xDFFFFFFF, 0xBFFFFFFF, 0x7FFFFFFF
		);
		
// ***** VARIABLES *****
		
		private var _map:Array; // format [y][x:0..31,32..63,...]
		private var _width:int, _height:int;
		
// ***** CONSTRUCTOR *****
		
		public function FlagMap(width:int = 64, height:int = 64)
		{
			var cntrX:int, cntrY:int;
			var wordWidth:int = (width + 31) / 32;
			var row:Array;
			
			_width = width;
			_height = height;
			_map = new Array(height);
			for (cntrY = 0; cntrY < height; ++cntrY) {
				row = new Array(wordWidth);
				for (cntrX = 0; cntrX < wordWidth; ++cntrX)
					row[cntrX] = 0;
				_map[cntrY] = row;
			}
		}
		
// ***** METHODS - Map Manipulation *****
		
		public function clear():void
		{
			var cntrX:int, cntrY:int;
			var rows:int = _map.length;
			var cols:int = _map[0].length;
			
			for (cntrY = 0; cntrY < rows; ++cntrY)
				for (cntrX = 0; cntrX < cols; ++cntrX)
					_map[cntrY][cntrX] = 0;
		}
		
		public function fill():void
		{
			var cntrX:int, cntrY:int;
			var rows:int = _map.length;
			var cols:int = _map[0].length;
			
			for (cntrY = 0; cntrY < rows; ++cntrY)
				for (cntrX = 0; cntrX < cols; ++cntrX)
					_map[cntrY][cntrX] = 0xFFFFFFFF;
		}
		
		public function setFlag(x:uint, y:uint, b:Boolean):void
		{
			var word:int = (x % _width) / 32;
			var bitIndex:int = (x % _width) % 32;
			if (b)
				_map[y % _height][word] |= BIT_MASKS[bitIndex];
			else
				_map[y % _height][word] &= INVERT_MASKS[bitIndex];
		}
		
		public function getFlag(x:uint, y:uint):Boolean
		{
			var word:int = (x % _width) / 32;
			var bitIndex:int = (x % _width) % 32;
			
			return ((_map[y % _height][word] & BIT_MASKS[bitIndex]) != 0);	
		}
		
		public function debugDump()
		{
			for (var cntrY:int = 0; cntrY < _width; ++cntrY) {
				var rowString:String = "";
				for (var cntrX:int = 0; cntrX < _width; ++cntrX) {
					if (getFlag(cntrX, cntrY))
						rowString = rowString + "T";
					else
						rowString = rowString + "F";
				}
				trace(rowString);				
			}
		}
	}
}