package com.blazinggames.tmod.common.map
{
	import flash.geom.Point;

	public class Pathfinder
	{
// ***** CONSTANTS *****
		
		public static const NO_MOVE:int = -1;
		public static const NORTH:int = 0;
		public static const EAST:int 1;
		public static const SOUTH:int = 2;
		public static const WEST:int = 3;
		
		private static const OBSTACLE:int = -1;
		private static const TARGET:int = -2;
		
// ***** VARIABLES *****
		
		private var _map:SimpleTileMap;
		private var _moveLimit:int;
		private var _bestWeight:int;
		private var _target:Point;
		
// ***** CONSTRUCTOR *****

		public function Pathfinder(w:int, h:int, limit:int)
		{
			_map = new SimpleTileMap(w, h, h);
			_moveLimit = limit;
		}

// ***** METHODS - Map management
		
		public function clear():void
		{
			_map.clearMap(_moveLimit);
		}
		
		public function placeObstacle(x:int, y:int):void
		{
			_map.setTile(x, y, OBSTACLE);
		}

		public function placeTarget(x:int, y:int):void
		{
			_map.setTile(x, y, TARGET);
		}
		
// ***** METHODS - pathfinding *****
		
		private function processPathTile(x:int, y:int):void
		{
			var tileWeight:int;
			var travelWeight:int = _map.getTile(x,y) + 1;
			if (travelWeight > _bestWeight)
				return;
			
			// process north
			if (y > 0) {
				tileWeight = _map.getTile(x, y-1);
				if (tileWeight == TARGET) {
					_bestWeight = travelWeight;
					_target = new Point(x, y-1);
					_map.setAttribute(x, y-1, NORTH);
					return;
				}
				if (tileWeight > travelWeight) {
					_map.setTile(x,y-1, travelWeight);
					_map.setAttribute(x, y-1, NORTH);
					processPathTile(x, y-1);
				}
			}
			
			// process west
			if (x > 0) {
				tileWeight = _map.getTile(x-1, y);
				if (tileWeight == TARGET) {
					_bestWeight = travelWeight;
					_target = new Point(x-1, y);
					_map.setAttribute(x-1, y, WEST);
					return;
				}
				if (tileWeight > travelWeight) {
					_map.setTile(x-1,y, travelWeight);
					_map.setAttribute(x-1, y, WEST);
					processPathTile(x-1, y);
				}
			}
	
	
			// process south
			if (y < (_map.getHeight()-1)) {
				tileWeight = _map.getTile(x, y+1);
				if (tileWeight == TARGET) {
					_bestWeight = travelWeight;
					_target = new Point(x, y+1);
					_map.setAttribute(x, y+1, SOUTH);
					return;
				}
				if (tileWeight > travelWeight) {
					_map.setTile(x,y+1, travelWeight);
					_map.setAttribute(x, y+1, SOUTH);
					processPathTile(x, y+1);
				}
			}
			
			// process east
			if (x < (_map.getWidth()-1) {
				tileWeight = _map.getTile(x+1, y);
				if (tileWeight == TARGET) {
					_bestWeight = travelWeight;
					_target = new Point(x+1, y);
					_map.setAttribute(x+1, y, EAST);
					return;
				}
				if (tileWeight > travelWeight) {
					_map.setTile(x+1,y, travelWeight);
					_map.setAttribute(x+1, y, EAST);
					processPathTile(x+1, y);
				}
			}
		}
		
		private function buildPathMap(x:int, y:int):void
		{
			_map.setTile(x, y, 0);
			_bestWeight = _moveLimit + 1;
			_target = null;
			processPathTile(x, y);
		}

		
		public function performStepSearch(startX:int, startY:int):int
		{
			var path:Array = performPathSearch(startX, startY);
			if (path.length == 0)
				return NO_MOVE;
			else if (path[0].y < startY)
				return NORTH;
			else if (path[0].y > startY)
				return SOUTH;
			else if (path[0].x < startX)
				return WEST;
			else
				return EAST;
		}
		
		public function performPathSearch(startX:int, startY:int):Array
		{
			buildPathMap(startX, startY);
			var start:Point = new Point(startX, startY);
			var path:Array = new Array();
			if (_target == null)
				return path;
			path.push(_target.clone());
			var dir:int;
			while ( ! _target.equals(start) ) {
				dir = _map.getAttributes(_target.x, _target.y);
				switch (dir) {
					case NORTH:
						_target.y++;
						break;
					case EAST:
						_target.x++;
						break;
					case SOUTH:
						_target.y--;
						break;
					case WEST:
						_target.x--;
						break;
					default:
						trace ("ERROR TRAVERSING BACKPATH");
						_target.x = start.x;
						_target.y = start.y;
						break;
				}
				path.push(_target.clone());
			}
			path.reverse();
			return path;
		}
		
	}
}