package com.blazinggames.tmod.common.map
{
	import flash.display.MovieClip;
	
	/**
	 * Your basic implementation of a tile renderer. This can be used as a stub
	 * class for a movie clip if there is only one tileset needed in the game 
	 * and can be easily overrided if more tilesets are needed. 
	 * Quite simply, the tileID is the frame, with an id <= 0 being invisible.
	 * 
	 * @author Billy D. Spelchan
	 */
	public class SimpleTileRenderer extends MovieClip implements ITileRenderer
	{
// ***** CONSTANTS *****
		
		public static const DEFAULT_TILE_WIDTH:int = 32;
		public static const DEFAULT_TILE_HEIGHT:int = 32;
		public static const DEFAULT_TILE_ID:int = 0;
		
// ***** VARIABLES *****
		
		private var _tileWidth:int, _tileHeight:int;
		private var _tileID:int;
		
// ***** CONSTRUCTOR *****
		
		public function SimpleTileRenderer(tileWidth:int = 32, tileHeight:int = 32, tileID = 0)
		{
			super();
			_tileWidth = tileWidth;
			_tileHeight= tileHeight;
			stop();
			setTileID(tileID);
		}
		
// ***** METHODS - ITileRenderer implementation *****
		
		public function getTileWidth():Number
		{
			return _tileWidth;
		}
		
		public function getTileHeight():Number
		{
			return _tileHeight;
		}
		
		public function getTileID():Number
		{
			return _tileID;
		}
		
		public function setTileID(id:int):void
		{
			_tileID = id;
			if (id > 0) {
				visible = true;
				gotoAndStop(id);
			} else {
				visible = false;
			}
		}
	}
}